// Copyright 2014 Concrete Software. All rights reserved.

uniform sampler2D texture;
// Direction of the main light in eye coordinates. Must be normalized.
uniform mediump vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform mediump vec3 pba_RimLightPosition;
//uniform lowp float pba_RimShininess;
// color (and intensity) for the diffuse light
uniform mediump vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform mediump vec3 pba_ambientColor;
// color (and intensity) for the specular component of the light
uniform mediump vec3 pba_specularColor;
// For when pins dissolve
uniform lowp float pba_dissolveAmount;
// Animation time.  Loops every 2*pi seconds
uniform highp float pba_time;

varying mediump vec3 var_Normal;
varying mediump vec3 var_TransformedVertex;
varying highp vec2 var_TexCoord;

bool shouldDissolve();
mediump float glow();

void main()
{
    if (shouldDissolve())
        discard;
	mediump vec3 normalizedNormal = normalize(var_Normal);
	mediump float diffuseIntensity = max(dot(normalizedNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(normalizedNormal, pba_RimLightPosition), 0.0);
	
	mediump vec3 eyePosition = normalize(var_TransformedVertex);
	mediump vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
	mediump float normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float specularComponent = pow(normalDotHalf, pba_KeyShininess);
    
    mediump float glow = glow();
    mediump vec3 texColor = texture2D(texture, var_TexCoord).rgb + vec3(glow,glow,0);
    mediump vec3 finalColor = (pba_ambientColor + pba_diffuseColor * diffuseIntensity) * texColor + (specularComponent * pba_specularColor);
    mediump float alpha = .7;
    
	gl_FragColor = vec4(finalColor*alpha,alpha);
}

//Loosly based on http://glsl.heroku.com/e#13448.0
mediump float glow() {
    highp float time = pba_time;
    mediump float glow = 0.;
	for(mediump float i=1.;i<2.0;i++){
		mediump float offsety=i+time*i*1.0;
		
		mediump float amt=1.-pow(abs(abs(var_TexCoord.x-1.5)*sin(0.25*3.*pow(var_TexCoord.x,3.0)+offsety)-var_TexCoord.y),0.02);
		glow+=amt*abs(tan(time*0.5+abs(var_TexCoord.y-0.5)*3.0));
		glow+=(amt+.03)*abs(tan(time+4.0+abs(var_TexCoord.y-1.)*13.0))*4.;
		glow+=(.01-amt)*abs(tan(time+3.0+abs(var_TexCoord.y)*9.0))*6.;
	}
    return abs(glow) * (pba_dissolveAmount * 20. + .3);
}


bool shouldDissolve() {
    if (pba_dissolveAmount == 0.)
        return false;
    if (pba_dissolveAmount == 1.)
        return true;
    highp float x = var_TexCoord.x / 100.;
    highp float sinx = sin(x);
    highp float sinxlarge = sinx * 40000.;
    highp float fractlarge = fract(sinxlarge);
    highp float dissolvePos = fractlarge;
	return abs(dissolvePos) < pba_dissolveAmount;
}